@@ -59,5 +59,9 @@
 #undef ARMA_USE_CXX11
 #endif
 
+// Rcpp has its own stream object which cooperates more nicely with R's i/o
+// And as of Armadillo 2.4.3, we can use this stream object as well 
+#define ARMA_DEFAULT_OSTREAM Rcpp::Rcout
+
 #endif
 
