@@ -1,5 +1,5 @@
-// Copyright (C) 2008-2010 NICTA (www.nicta.com.au)
-// Copyright (C) 2008-2010 Conrad Sanderson
+// Copyright (C) 2008-2012 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2012 Conrad Sanderson
 // 
 // This file is part of the Armadillo C++ library.
 // It is provided without any warranty of fitness
@@ -18,31 +18,41 @@
 // operator %, which we define it to do a schur product (element-wise multiplication)
 
 
-//! element-wise multiplication of Base objects with same element type
+//! element-wise multiplication of user-accessible Armadillo objects with same element type
 template<typename T1, typename T2>
 arma_inline
-const eGlue<T1, T2, eglue_schur>
+typename
+enable_if2
+  <
+  is_arma_type<T1>::value && is_arma_type<T2>::value && is_same_type<typename T1::elem_type, typename T2::elem_type>::value,
+  const eGlue<T1, T2, eglue_schur>
+  >::result
 operator%
   (
-  const Base<typename T1::elem_type,T1>& X,
-  const Base<typename T1::elem_type,T2>& Y
+  const T1& X,
+  const T2& Y
   )
   {
   arma_extra_debug_sigprint();
   
-  return eGlue<T1, T2, eglue_schur>(X.get_ref(), Y.get_ref());
+  return eGlue<T1, T2, eglue_schur>(X, Y);
   }
 
 
 
-//! element-wise multiplication of Base objects with different element types
+//! element-wise multiplication of user-accessible Armadillo objects with different element types
 template<typename T1, typename T2>
 inline
-const mtGlue<typename promote_type<typename T1::elem_type, typename T2::elem_type>::result, T1, T2, glue_mixed_schur>
+typename
+enable_if2
+  <
+  (is_arma_type<T1>::value && is_arma_type<T2>::value && (is_same_type<typename T1::elem_type, typename T2::elem_type>::value == false)),
+  const mtGlue<typename promote_type<typename T1::elem_type, typename T2::elem_type>::result, T1, T2, glue_mixed_schur>
+  >::result
 operator%
   (
-  const Base< typename force_different_type<typename T1::elem_type, typename T2::elem_type>::T1_result, T1>& X,
-  const Base< typename force_different_type<typename T1::elem_type, typename T2::elem_type>::T2_result, T2>& Y
+  const T1& X,
+  const T2& Y
   )
   {
   arma_extra_debug_sigprint();
@@ -54,7 +64,7 @@ operator%
   
   promote_type<eT1,eT2>::check();
   
-  return mtGlue<out_eT, T1, T2, glue_mixed_schur>( X.get_ref(), Y.get_ref() );
+  return mtGlue<out_eT, T1, T2, glue_mixed_schur>( X, Y );
   }
 
 
